/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.util.HashMap;
import java.util.Map;

import jline.console.completer.Completer;

import org.apache.commons.cli.Options;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.shell.Shell;

/**
 * A petals CLI Command is executed inside a petals CLI shell.
 * @author Sebastien Andre - EBM WebSourcing
 */
public abstract class AbstractCommand implements Command {


    private final String name;

    private String usage = "";

    private String description = "";

    private String optionsDescription;

    private Options options;

    protected final HashMap<String, Completer> completers = new HashMap<String, Completer>();

    @Override
	public Options getOptions() {
        return this.options;
    }

    @Override
	public String getOptionsDescription() {
        return this.optionsDescription;
    }

    protected void setOptionsDescription(String optionsDescription) {
        this.optionsDescription = optionsDescription;
    }

    private Shell shell;

    /**
     * Initialize the command by inferring its name from the class name
     */
    public AbstractCommand() {
        this.name = this.getClass().getSimpleName().toLowerCase();
        this.options = this.createOptions();
    }

    /**
     * Create a new command
     *
     * @param cmdName
     */
    public AbstractCommand(final String cmdName) {
        this.name = cmdName;
        this.shell = null;
        this.options = this.createOptions();
    }

    /**
     * Get the name of the command from its class name.
     *
     * @return the name of the Class in low cases. Not <code>null</code>.
     */
    @Override
	public final String getName() {
        return this.name;
    }

    /**
     * Get the usage message for this command
     *
     * @return A string describing command's arguments
     */
    @Override
	public final String getUsage() {
        return this.usage;
    }

    /**
     * Get the description of this command
     *
     * @return A string describing the purpose and use of this command
     */
    @Override
	public final String getDescription() {
        return this.description;
    }

    /**
     * Get the shell this command is bound to
     *
     * @return a petals CLI shell
     */
    @Override
	public final Shell getShell() {
        return this.shell;
    }

    @Override
	public final void setShell(final Shell shell) {
        this.shell = shell;
    }

    /**
     * Set the usage message
     *
     * @param usage
     */
    protected final void setUsage(String usage) {
        this.usage = usage;
    }

    /**
     * Set the description
     *
     * @param description
     */
    protected final void setDescription(String description) {
        this.description = description;
    }

    /**
     * Check the number of arguments to be greater than a specified number
     *
     * @param args
     *            the array of arguments
     * @param min
     *            the minimal number of elements args must contain
     * @return <code>true</code> if args is null or contains less than min
     *         elements
     */
    public final boolean checkArguments(String[] args, int min) throws CommandException {
        return (args != null && args.length >= min);
    }

    /**
     * Check the number of arguments according to a minimum and a maximum number
     *
     * @param args
     *            the array of arguments
     * @param min
     *            the minimal number of elements args must contain
     * @param max
     *            the maximal number of elements args must contain
     * @return <code>true</code> if args is null or contained in the interval
     */
    public final boolean checkArguments(String[] args, int min, int max) throws CommandException {
        return (this.checkArguments(args, min) && args.length <= max);
    }

    @Override
	public final void resetOptions() {
        // reset options
        this.options = this.createOptions();
    }

    @Override
	public abstract void execute(String[] args) throws CommandException;

    /**
     * Create the options specific to the command
     */
    public Options createOptions() {
        return new Options();
    }

    /**
     * Get the completer for the unique argument (Only one argument is allowed for a command)
     *
     * @return the completer for the unique argument or null if there is no completer
     */
    @Override
	public Completer getDefaultCompleter() {
        return null;
    }

    /**
     * Get the completers for the options
     *
     * @return a map with short option as key and corresponding completer as
     *         value (If an option has no completer, the corresponding short option
     *         cannot be in the map)
     */
    @Override
	public Map<String, Completer> getOptionCompleters() {
        return this.completers;
    }
}
