/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import jline.console.completer.Completer;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.exception.ShellException;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.connection.ConnectionHelper;
import org.ow2.petals.cli.connection.PreferenceFileException;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class Connect extends AbstractCommand {

    /**
     * The 'yes' flag option to skip confirmation
     */
    public static final String YESFLAG_SHORT_OPTION = "y";

    private static final Option YESFLAG_OPTION = OptionBuilder.hasArg(false)
            .withDescription("A flag to skip confirmation.").create(YESFLAG_SHORT_OPTION);

    public Connect() {
        super();
        StringBuilder usage = new StringBuilder("usage: connect ");
        ConnectionHelper.addConnectionUsage(usage, true);
        this.setUsage(usage.toString());
        this.setDescription("Connect to a Petals ESB node");
    }

    @Override
    public void execute(final String[] args) throws CommandException {
        final Shell shell = this.getShell();

        try {
            final CommandLineParser parser = new PosixParser();
            final CommandLine cmd = parser.parse(this.getOptions(), args);

            final ConnectionParameters connectionParameters = ConnectionHelper
                    .parseConnectionParameters(cmd, true);

            if (cmd.getOptions().length == 0) {
                final String confirmationQuestion;
                final String host = connectionParameters.getHost();
                final int port = connectionParameters.getPort();
                if (connectionParameters instanceof AuthenticatedConnectionParameters) {
                    final String username = ((AuthenticatedConnectionParameters) connectionParameters)
                            .getUsername();
                    confirmationQuestion = String.format(
                            "Would you like to connect to %s:*****@%s:%d? (y/n) ", username, host, port);
                } else {
                    confirmationQuestion = String.format(
                            "Would you like to connect to %s:%d? (y/n) ", host, port);
                }
                if (cmd.hasOption(YESFLAG_SHORT_OPTION) || shell.confirms(confirmationQuestion)) {
                    this.getShell().setConnectionParameters(
                            ConnectionHelper.connect(connectionParameters, shell,
                                    cmd.hasOption(YESFLAG_SHORT_OPTION)));
                }

            } else {
                final AuthenticatedConnectionParameters authenticatedConnectionParameters = ConnectionHelper
                        .connect(connectionParameters, shell, cmd.hasOption(YESFLAG_SHORT_OPTION));
                this.getShell().setConnectionParameters(authenticatedConnectionParameters);
                if(authenticatedConnectionParameters != null) {
                    String confirmation = String.format("Connected on %s:%d with '%s'",
                            authenticatedConnectionParameters.getHost(), authenticatedConnectionParameters.getPort(),
                            authenticatedConnectionParameters.getUsername());
                    shell.getPrintStream().println(confirmation);
                }
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ContainerAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final IOException e) {
            throw new CommandException(this, e);
        } catch (final PreferenceFileException e) {
            throw new CommandException(this, e);
        } catch (final ShellException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options getOptions() {
        Options options = new Options();
        ConnectionHelper.addConnectionOptions(options);
        options.addOption(YESFLAG_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        try {
            return ConnectionHelper.getConnectionOptionCompleters();
        } catch (PreferenceFileException pfe) {
            // no available completion
            return new HashMap<String, Completer>();
        }
    }
}
