/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.ow2.petals.cli.Constants.CommonOption.URL_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import jline.console.completer.Completer;
import jline.console.completer.FileNameCompleter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.conf.PetalsAdminConfiguration;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.cli.shell.util.ArtifactUrlRewriter;
import org.ow2.petals.cli.shell.util.Utils;

/**
 * This command deploys a JBI artifact on petals ESB.
 * @author Sebastien Andre - EBM WebSourcing
 */
public class Deploy extends AbstractCommand {

    public final static String PROPERTIES_FILE_SHORT_OPTION = "f";

    public final static String PROPERTIES_FILE_LONG_OPTION = "file";

    public final static String INLINED_PROPERTIES_SHORT_OPTION = "D";

    protected static final Option INLINED_PROPERTIES_OPTION = OptionBuilder
            .withArgName("property1=value1,property2=value2")
            .hasArgs()
            .withValueSeparator(',')
            .withDescription(
                    "List of '<property-name>=<property-value>', separated by a comma character, where <property-name> is the name of the property to configure with <property-value>. This argument is used only if the artifact is a component. This argument is exclusive with <configuration-file>.")
            .create(INLINED_PROPERTIES_SHORT_OPTION);

    protected static final Option PROPERTIES_FILE_OPTION = OptionBuilder
            .hasArgs(1)
            .withArgName("configuration-file")
            .withLongOpt(PROPERTIES_FILE_LONG_OPTION)
            .withDescription(
                    "The local file name or the URL of the properties file used to configure the artifact. This argument is used only if the artifact is a component. This argument is exclusive with <configuration-properties>.")
            .create(PROPERTIES_FILE_SHORT_OPTION);

    /**
     * The bulk option
     */
    public final static String BULK_SHORT_OPTION = Constants.CommonOption.BULK_SHORT_OPTION;

    private final static String BULK_LONG_OPTION = Constants.CommonOption.BULK_LONG_OPTION;

    protected static final Option BULK_OPTION = OptionBuilder.hasArgs(1).withArgName("url")
            .withLongOpt(BULK_LONG_OPTION)
            .withDescription("Deploy and start artifacts found in the given directory")
            .create(BULK_SHORT_OPTION);

    private final HashMap<String, Completer> completers = new HashMap<String, Completer>();

    public Deploy() {
        super();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Deploy and start a JBI artifact");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        FileNameCompleter c = new FileNameCompleter();
        this.completers.put(BULK_SHORT_OPTION, c);
        this.completers.put(URL_SHORT_OPTION, c);
        this.completers.put(PROPERTIES_FILE_SHORT_OPTION, c);
    }

    @Override
    public void execute(String[] args) throws CommandException {

        final CommandLineParser parser = new PosixParser();
        try {
            final ArtifactAdministration artifactAdministration = PetalsAdministrationFactory.newInstance(
                    new PetalsAdminConfiguration(new ArtifactUrlRewriter(this.getShell()))).newArtifactAdministration();

            final CommandLine cli = parser.parse(this.getOptions(), args);
            URL url;
            // bulk deployment
            if (cli.hasOption(BULK_SHORT_OPTION)) {
                if (!this.checkArguments(args, 2, 2)) {
                    throw new CommandTooManyArgumentsException(this, args);
                }
                url = Utils.toURL(cli.getOptionValue(BULK_SHORT_OPTION));
                if (url.getProtocol().equals("file")) {
                    File filePath = new File(url.getFile());
                    artifactAdministration.deployAndStartArtifacts(Utils.getUrls(filePath));
                } else {
                    throw new CommandException(this,
                            "Directory deployment is only allowed for file protocol");
                }
            } else if (cli.hasOption(URL_SHORT_OPTION)) {
                url = Utils.toURL(cli.getOptionValue(URL_SHORT_OPTION));
                if (cli.hasOption(PROPERTIES_FILE_SHORT_OPTION)) {
                    if (!this.checkArguments(args, 2, 4)) {
                        throw new CommandTooManyArgumentsException(this, args);
                    }
                    String propertiesFile = cli.getOptionValue('f');
                    artifactAdministration.deployAndStartArtifact(url, Utils.toURL(propertiesFile));
                } else if (cli.hasOption(INLINED_PROPERTIES_SHORT_OPTION)) {
                    final Properties configurationProperties = new Properties();
                    final String[] properties = cli.getOptionValues(INLINED_PROPERTIES_SHORT_OPTION);
                    for (String propertyEntry : properties) {
                        String[] property = propertyEntry.split("=");
                        if (property.length != 2) {
                            throw new CommandUnparsableArgumentException(this,
                                    INLINED_PROPERTIES_OPTION,
                                    propertyEntry);
                        }
                        configurationProperties.put(property[0], property[1]);
                    }
                    artifactAdministration.deployAndStartArtifact(url, configurationProperties);
                } else {
                    if (!this.checkArguments(args, 2, 2)) {
                        throw new CommandTooManyArgumentsException(this, args);
                    }
                    artifactAdministration.deployAndStartArtifact(url);
                }
            }
            else {
                final List missingOptions = new ArrayList();
                missingOptions.add(BULK_SHORT_OPTION);
                missingOptions.add(URL_SHORT_OPTION);
                throw new CommandMissingOptionsException(this, missingOptions);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ArtifactAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final IOException e) {
            throw new CommandException(this, e);
        } catch (final DuplicatedServiceException e) {
            throw new CommandException(this, e);
        } catch (final MissingServiceException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options createOptions() {
        Options options = new Options();
        OptionGroup optionGroup = new OptionGroup();
        optionGroup.addOption(BULK_OPTION);
        optionGroup.addOption(URL_OPTION);

        OptionGroup propertiesOptionGroup = new OptionGroup();
        propertiesOptionGroup.addOption(PROPERTIES_FILE_OPTION);
        propertiesOptionGroup.addOption(INLINED_PROPERTIES_OPTION);
        options.addOptionGroup(optionGroup);
        options.addOptionGroup(propertiesOptionGroup);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        return this.completers;
    }

}
