/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.util.Map;
import java.util.TreeMap;

import jline.console.completer.Completer;
import jline.console.completer.StringsCompleter;

import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;

/**
 * This command print help about other commands in the shell.
 * @author Sebastien Andre - EBM WebSourcing
 */
public class Help extends AbstractCommand {

    public Help() {
        super();
        this.setUsage(this.getName() + " <command_name>");
        this.setDescription("Display this help message or help for a specific command");
    }

    @Override
    public void execute(String[] args) throws CommandException {
        if (!this.checkArguments(args, 0, 1)) {
            throw new CommandTooManyArgumentsException(this, args);
        }
        String message;

        if (args.length == 0) {
            message = this.getHelp();

        } else if (this.getShell().getCommands().containsKey(args[0])) {
            message = this.getHelpCommand(args[0]);

        } else {
            throw new CommandException(this, args[0] + ": unknown command");
        }

        this.getShell().getPrintStream().println(message);
    }

    private String formatTitle(String title) {
        return String.format("%n%s:%n", title);
    }

    private String formatInfo(String info) {
        return String.format("   %s%n", info);
    }

    private String formatHelp(String name, String desc) {
        if (name.length() <= 8)
            return String.format("   %-10s  %s.%n", name, desc);

        return String.format("   %s%n               %s.%n", name, desc);
    }

    private String getHelp() {
        TreeMap<String, String> descs = new TreeMap<String, String>();

        for (final Command cmd : this.getShell().getCommands().values()) {
            String name = cmd.getName();
            descs.put(name, cmd.getDescription());
        }

        StringBuffer buffer = new StringBuffer();

        buffer.append(this.formatTitle("Available commands"));

        for (Map.Entry<String, String> entry : descs.entrySet()) {
            buffer.append(this.formatHelp(entry.getKey(), entry.getValue()));
        }

        buffer.append(this.formatTitle("For help on a specific command type"));
        buffer.append(this.formatInfo("help <command>"));

        return buffer.toString();
    }

    private String getHelpCommand(String name) {
        final Command cmd = this.getShell().getCommands().get(name);

        String usage = cmd.getUsage();
        String description = cmd.getDescription();
        String optionDescription = cmd.getOptionsDescription();
        String format;
        if (StringHelper.isNullOrEmpty(optionDescription)) {
            format = String.format("%nUSAGE:%n %s %s%n%nDESCRIPTION:%n %s%n", name, usage,
                    description);
        } else {
            format = String.format(
                    "%nUSAGE:%n %s %s%n%nDESCRIPTION:%n %s%n%nOPTIONS DESCRIPTION:%n%s%n", name,
                    usage, description, optionDescription);
        }
        return format;
    }

    @Override
    public Completer getDefaultCompleter() {
        return new StringsCompleter(this.getShell().getCommands().keySet());
    }

}
