/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.PrintStream;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.RegistryAdministration;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.api.exception.RegistryAdministrationException;
import org.ow2.petals.admin.registry.Endpoint;
import org.ow2.petals.admin.registry.RegistryView;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;

/**
 * This command registry-list a petals ESB container.
 * @author Sebastien Andre - EBM WebSourcing
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class ListRegistry extends AbstractCommand {

    private static final String DEFAULT_REGEX_OPTION_VALUE = ".*";

    /**
     * Options supported on the command line
     */
    private static final Options OPTIONS = new Options();

    /**
     * The endpoint name regular expression option
     */
    private static final String ENDPOINT_NAME_SHORT_OPTION = "e";
    private static final String ENDPOINT_NAME_LONG_OPTION = "endpoint";
    private static final String ENDPOINT_NAME_ARG_NAME = "endpoint";
    private static final Option ENDPOINT_NAME_OPTION = OptionBuilder.
            hasArgs(1).withArgName(ENDPOINT_NAME_ARG_NAME)
            .withLongOpt(ENDPOINT_NAME_LONG_OPTION)
            .withDescription("Endpoint name regular expression.")
            .create(ENDPOINT_NAME_SHORT_OPTION);

    /**
     * The service name regular expression option
     */
    private static final String SERVICE_NAME_SHORT_OPTION = "s";
    private static final String SERVICE_NAME_LONG_OPTION = "service";
    private static final String SERVICE_NAME_ARG_NAME = "service";
    private static final Option SERVICE_NAME_OPTION = OptionBuilder.
            hasArgs(1).withArgName(SERVICE_NAME_ARG_NAME)
            .withLongOpt(SERVICE_NAME_LONG_OPTION)
            .withDescription("Service name regular expression.")
            .create(SERVICE_NAME_SHORT_OPTION);

    /**
     * The interface name regular expression option
     */
    private static final String INTERFACE_NAME_SHORT_OPTION = "i";
    private static final String INTERFACE_NAME_LONG_OPTION = "interface";
    private static final String INTERFACE_NAME_ARG_NAME = "interface";
    private static final Option INTERFACE_NAME_OPTION = OptionBuilder.
            hasArgs(1).withArgName(INTERFACE_NAME_ARG_NAME)
            .withLongOpt(INTERFACE_NAME_LONG_OPTION)
            .withDescription("Interface name regular expression.")
            .create(INTERFACE_NAME_SHORT_OPTION);

    public ListRegistry() {
        super("registry-list");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("List the entries of the registry");
    }

    @Override
    public void execute(String[] args) throws CommandException {
        try {
            CommandLineParser parser = new PosixParser();
            CommandLine cmd = parser.parse(this.getOptions(), args);

            String endpointNameRegex = getRegexOption(cmd, ENDPOINT_NAME_SHORT_OPTION);
            String serviceNameRegex = getRegexOption(cmd, SERVICE_NAME_SHORT_OPTION);
            String interfaceNameRegex = getRegexOption(cmd, INTERFACE_NAME_SHORT_OPTION);

            RegistryAdministration registryAdministration = PetalsAdministrationFactory
                    .newInstance().newRegistryAdministration();
            RegistryView registryView = registryAdministration.getRegistryContent(endpointNameRegex, serviceNameRegex, interfaceNameRegex);

            StringBuilder registryContent = new StringBuilder("Endpoints:\n");

            List<Endpoint> endpoints = registryView.getAllEndpoints();
            for(Endpoint endpoint : endpoints) {
                String endpointInformation = String.format("%s: %s,%s,%s\n",
                        endpoint.getEndpointName(), endpoint.getContainerName(),
                        endpoint.getComponentName(), endpoint.getEndpointType());
                registryContent.append(endpointInformation);
            }

            registryContent.append("Services:\n");
            Map<String, List<Endpoint>> endpointsByServiceName = registryView.getListOfEndpointsByServiceName();
            for(String serviceName : endpointsByServiceName.keySet()) {
                registryContent.append(serviceName + ":\n");
                for(Endpoint endpoint : endpointsByServiceName.get(serviceName)) {
                    registryContent.append(endpoint.getEndpointName() + " ");
                }
                registryContent.append("\n");
            }

            registryContent.append("Interfaces:\n");
            Map<String, List<Endpoint>> endpointsByInterfaceName = registryView.getListOfEndpointsByInterfaceName();
            for(String interfaceName : endpointsByInterfaceName.keySet()) {
                registryContent.append(interfaceName + ":\n");
                for(Endpoint endpoint : endpointsByInterfaceName.get(interfaceName)) {
                    registryContent.append(endpoint.getEndpointName() + " ");
                }
                registryContent.append("\n");
            }

            PrintStream out = this.getShell().getPrintStream();
            out.println(registryContent);
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final RegistryAdministrationException e) {
            throw new CommandException(this, e);
        } catch (final DuplicatedServiceException e) {
            throw new CommandException(this, e);
        } catch (final MissingServiceException e) {
            throw new CommandException(this, e);
        }
    }

    private static final String getRegexOption(CommandLine cmd, String option) {
        String endpointNameRegex;
        if (cmd.hasOption(option)) {
            endpointNameRegex = cmd.getOptionValue(option);
        } else {
            endpointNameRegex = DEFAULT_REGEX_OPTION_VALUE;
        }
        return endpointNameRegex;
    }

    @Override
    public Options createOptions() {
        Options options = new Options();
        options.addOption(ENDPOINT_NAME_OPTION);
        options.addOption(SERVICE_NAME_OPTION);
        options.addOption(INTERFACE_NAME_OPTION);
        return options;
    }
}
