/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;

import jline.console.completer.Completer;
import jline.console.completer.FileNameCompleter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.admin.topology.Subdomain;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.ConnectionParameters;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class TopologyList extends AbstractCommand {

    public final static String PASSPHRASE_SHORT_OPTION = "p";

    public final static String PASSPHRASE_LONG_OPTION = "pass-phrase";

    protected static final Option PASSPHRASE_OPTION = OptionBuilder.isRequired(false).hasArgs(1)
            .withArgName("pass-phrase").withLongOpt(PASSPHRASE_LONG_OPTION)
            .withDescription("The security pass-phrase to get critical information.")
            .create(PASSPHRASE_SHORT_OPTION);

    public TopologyList() {
        super("topology-list");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Get information about the topology this node is part of.");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
        FileNameCompleter c = new FileNameCompleter();
        this.completers.put(PASSPHRASE_SHORT_OPTION, c);
    }

    @Override
    public void execute(String[] args) throws CommandException {
        

        final CommandLineParser parser = new PosixParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            
            final String passPhrase;
            if (cli.hasOption(PASSPHRASE_SHORT_OPTION)) {
                // The pass phrase is provided as argument option
                passPhrase = cli.getOptionValue(PASSPHRASE_SHORT_OPTION);
            } else if (cli.getArgList().size() > 0) {
                throw new CommandBadArgumentNumberException(this);
            } else {
                // We check that the pass phrase is set at alias level
                final ConnectionParameters connectionParameters = this.getShell().getConnectionParameters();

                if (connectionParameters == null) {
                    passPhrase = null;
                }
                else if (connectionParameters instanceof AuthenticatedConnectionParameters) {
                    passPhrase = ((AuthenticatedConnectionParameters)connectionParameters).getPassPhrase();
                }
                else {
                    passPhrase = null;
                }
            }
            
            ContainerAdministration containerAdministration = PetalsAdministrationFactory
                    .newInstance().newContainerAdministration();
            Domain domain = containerAdministration.getTopology(passPhrase);
            if (domain == null) {
                throw new CommandException(this, "No Petals domain found");
            }
            StringBuilder topologyInformation = new StringBuilder(String.format("Domain: %s%n",
                    domain.getName()));

            for (Subdomain subdomain : domain.getSubdomains()) {
                topologyInformation.append(String.format("Subdomain: %s%n", subdomain.getName()));

                for (Container container : subdomain.getContainers()) {
                    String containerInformation = String.format(
                            "* Container: %s%n** Type: %s%n** Host: %s%n",
                            container.getContainerName(), container.getNodeType().toString()
                                    .toLowerCase(), container.getHost());
                    topologyInformation.append(containerInformation);

                    HashMap<PortType, Integer> ports = container.getPorts();
                    for (PortType portType : ports.keySet()) {
                        topologyInformation.append(String.format("** %s: %d%n", portType,
                                ports.get(portType)));
                    }

                    final String jmxUser = container.getJmxUsername();
                    topologyInformation.append(String.format("** JMX username: %s%n",
                            (jmxUser == null || jmxUser.isEmpty()) ? "*********" : jmxUser));
                    final String jmxPassword = container.getJmxPassword();
                    topologyInformation.append(String.format("** JMX password: %s%n",
                            (jmxPassword == null || jmxPassword.isEmpty()) ? "*********"
                                    : jmxPassword));

                }
            }

            PrintStream out = this.getShell().getPrintStream();
            out.println(topologyInformation);

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption());
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions());
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final DuplicatedServiceException e) {
            throw new CommandException(this, e);
        } catch (final MissingServiceException e) {
            throw new CommandException(this, e);
        } catch (final ContainerAdministrationException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options createOptions() {
        Options options = new Options();
        options.addOption(PASSPHRASE_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        return this.completers;
    }
}
