/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.io.Writer;
import java.util.Properties;

import org.junit.Rule;
import org.junit.Test;
import org.junit.experimental.categories.Category;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.exception.DuplicatedCommandException;
import org.ow2.petals.cli.connection.ConnectionHelper;
import org.ow2.petals.cli.connection.PreferenceFileManager;
import org.ow2.petals.cli.connection.PreferenceFileManagerTest;
import org.ow2.petals.cli.connection.UnitTestsRequiringSetEnvVar;
import org.ow2.petals.cli.shell.PetalsFileScriptShell;
import org.ow2.petals.cli.shell.PetalsInteractiveCliTest;

/**
 * Unit test of {@link Connect}
 *
 * @author Christophe DENEUX - Linagora
 *
 */
public class ConnectTest extends AbstractArtifactCommandTest {

    /**
     * Assertion exception used by threads
     */
    private volatile AssertionError assertion;

    @Rule
    public final TemporaryFolder tempFolder = new TemporaryFolder();

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Connect command = new Connect();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_00() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Connect command = new Connect();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>username and password are asked,</li>
     * <li>after to have entered username and password, the command is executed
     * displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_01() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            final String error0 = errBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("An error occurs", error0.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));
                            // Enter the command
                            pos.write(( "connect -a " + alias + "\n" ).getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error1 = errBAIS.toString();
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertTrue("An error message is print", error1.isEmpty());
                            assertTrue("Username is not asked",
                                    output2.endsWith(ConnectionHelper.USERNAME_QUESTION));

                            // Enter username
                            pos.write( "petals\n".getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error2 = errBAIS.toString().substring(error1.length());
                            final String output3 = outBAIS.toString().substring(
                                    output1.length() + output2.length());
                            assertTrue("An error message is print", error2.isEmpty());
                            assertTrue("Password is not asked",
                                    output3.endsWith(ConnectionHelper.PASSWORD_QUESTION));

                            // Enter password
                            pos.write( "petals\n".getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error3 = errBAIS.toString().substring(
                                    error1.length() + error2.length());
                            final String output4 = outBAIS.toString().substring(
                                    output1.length() + output2.length() + output3.length());
                            assertTrue("An error message is print", error3.isEmpty());
                            assertTrue("Unexpected prompt",
                                    output4.endsWith("petals-cli@" + host + ":" + port + ">"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-C" });

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);

            // Remove the preference file
            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password,</li>
     * <li>with yes flag as command option,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>username and password are not asked,</li>
     * <li>an error occurs about missing username and password</li>
     * </ul>
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_02() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));
                            // Enter the command
                            pos.write(String.format("connect -a %s -y%n", alias).getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error1 = errBAIS.toString();
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertTrue("No error occurs", error1.endsWith(String.format("%s%n",
                                    Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD)));
                            assertTrue("Petals CLI is connected", output2.contains("petals-cli>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-C" });

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);

            // Remove the preference file
            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password,</li>
     * <li>with yes flag on the command line,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>username and password are not asked,</li>
     * <li>an error occurs about missing username and password</li>
     * </ul>
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_03() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));
                            // Enter the command
                            pos.write(String.format("connect -a %s%n", alias).getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error1 = errBAIS.toString();
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertTrue("No error occurs", error1.endsWith(String.format("%s%n",
                                    Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD)));
                            assertTrue("Petals CLI is connected", output2.contains("petals-cli>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-y", "-C" });

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);

            // Remove the preference file
            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with a username and without password,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing password and it is not asked</li>
     * <li>the interactive shell ends,</li>
     * <li>the exit code is correctly set to 1.</li>
     * </ul>
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_07() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        properties.setProperty(alias + ".username", "petals");
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            final String error1 = errBAIS.toString();
                            assertTrue("Something is printed on output", output1.isEmpty());
                            assertFalse("No error occurs", error1.isEmpty());
                            assertTrue("Missing alias name containing the error",
                                    error1.contains(alias));
                            assertFalse("Prompt is displayed", error1.contains("petals-cli>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-C" });

                        assertEquals("invalid exit code", 1, exitCode);
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);

            // Remove the preference file
            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without a username and with a password,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing username and it is not asked</li>
     * <li>the interactive shell ends,</li>
     * <li>the exit code is correctly set to 1.</li>
     * </ul>
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_08() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        properties.setProperty(alias + ".password", "petals");
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            final String error1 = errBAIS.toString();
                            assertTrue("Something is printed on output", output1.isEmpty());
                            assertFalse("No error occurs", error1.isEmpty());
                            assertTrue("Missing alias name containing the error",
                                    error1.contains(alias));
                            assertFalse("Prompt is displayed", error1.contains("petals-cli>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-C" });

                        assertEquals("invalid exit code", 1, exitCode);
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);

            // Remove the preference file
            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password,</li>
     * <li>without yes flag,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing username and password.
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_04() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, IOException {

        final PrintStream standardConsoleError = System.err;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(ConnectionHelper.ALIAS_SHORT_OPTION);
            pwScript.write(alias);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final PetalsFileScriptShell shell = new PetalsFileScriptShell(
                    petalsCliScript.getAbsolutePath(), null, false, false);
            shell.registersCommand(command);
            command.setShell(shell);

            shell.run();

            final String error = errBAIS.toString();
            assertFalse("No error message print", error.isEmpty());
            assertTrue("It's not the expected error message.",
                    error.contains(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD));

        } finally {
            System.setErr(standardConsoleError);

            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password,</li>
     * <li>with the yes flag set as command option,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing username and password.
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_05() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, IOException {

        final PrintStream standardConsoleError = System.err;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(ConnectionHelper.ALIAS_SHORT_OPTION);
            pwScript.write(alias);
            pwScript.write(" -");
            pwScript.write(Connect.YESFLAG_SHORT_OPTION);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final PetalsFileScriptShell shell = new PetalsFileScriptShell(
                    petalsCliScript.getAbsolutePath(), null, false, false);
            shell.registersCommand(command);
            command.setShell(shell);

            shell.run();

            final String error = errBAIS.toString();
            assertFalse("No error message print", error.isEmpty());
            assertTrue("It's not the expected error message.",
                    error.contains(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD));

        } finally {
            System.setErr(standardConsoleError);

            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password,</li>
     * <li>with the yes flag set on the command line,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing username and password.
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArgumentsError_06() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, IOException {

        final PrintStream standardConsoleError = System.err;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(ConnectionHelper.ALIAS_SHORT_OPTION);
            pwScript.write(alias);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final PetalsFileScriptShell shell = new PetalsFileScriptShell(
                    petalsCliScript.getAbsolutePath(), null, false, true);
            shell.registersCommand(command);
            command.setShell(shell);

            shell.run();

            final String error = errBAIS.toString();
            assertFalse("No error message print", error.isEmpty());
            assertTrue("It's not the expected error message.",
                    error.contains(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD));

        } finally {
            System.setErr(standardConsoleError);

            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with username/password,</li>
     * <li>without yes flag,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArguments_00() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, IOException {

        final PrintStream standardConsoleError = System.err;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        properties.setProperty(alias + ".username", "petals");
        properties.setProperty(alias + ".password", "petals");
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(ConnectionHelper.ALIAS_SHORT_OPTION);
            pwScript.write(alias);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final PetalsFileScriptShell shell = new PetalsFileScriptShell(
                    petalsCliScript.getAbsolutePath(), null, false, false);
            shell.registersCommand(command);
            command.setShell(shell);

            shell.run();

            final String error = errBAIS.toString();
            assertTrue("An error message is printed", error.isEmpty());
            assertEquals("Unexpected exit code", 0, shell.getExitStatus());

        } finally {
            System.setErr(standardConsoleError);

            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with username/password,</li>
     * <li>with a yes flag set as command option,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArguments_01() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, IOException {

        final PrintStream standardConsoleError = System.err;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        properties.setProperty(alias + ".username", "petals");
        properties.setProperty(alias + ".password", "petals");
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(ConnectionHelper.ALIAS_SHORT_OPTION);
            pwScript.write(alias);
            pwScript.write(" -");
            pwScript.write(Connect.YESFLAG_SHORT_OPTION);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final PetalsFileScriptShell shell = new PetalsFileScriptShell(
                    petalsCliScript.getAbsolutePath(), null, false, false);
            shell.registersCommand(command);
            command.setShell(shell);

            shell.run();

            final String error = errBAIS.toString();
            assertTrue("An error message is printed", error.isEmpty());
            assertEquals("Unexpected exit code", 0, shell.getExitStatus());

        } finally {
            System.setErr(standardConsoleError);

            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with username/password,</li>
     * <li>with the yes flag set on the command line,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    @Category(UnitTestsRequiringSetEnvVar.class)
    public void testArguments_02() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, IOException {

        final PrintStream standardConsoleError = System.err;

        // Create the preference file
        assertNotNull("Env var PREFERENCES_FILE_FOR_TEST no set",
                PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final File preferenceFile = new File(PreferenceFileManagerTest.PREFERENCES_FILE_FOR_TEST);
        final Properties properties = new Properties();
        final String alias = "test-alias";
        properties.setProperty(PreferenceFileManager.DEFAULT_ALIAS_PROPERTY, alias);
        final String host = "localhost";
        properties.setProperty(alias + ".host", host);
        final String port = "7700";
        properties.setProperty(alias + ".port", port);
        properties.setProperty(alias + ".username", "petals");
        properties.setProperty(alias + ".password", "petals");
        final OutputStream preferenceFileOS = new FileOutputStream(preferenceFile);
        try {
            properties.store(preferenceFileOS, "test property list");
        } finally {
            preferenceFileOS.close();
        }
        PreferenceFileManager.reset();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(ConnectionHelper.ALIAS_SHORT_OPTION);
            pwScript.write(alias);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final PetalsFileScriptShell shell = new PetalsFileScriptShell(
                    petalsCliScript.getAbsolutePath(), null, false, true);
            shell.registersCommand(command);
            command.setShell(shell);

            shell.run();

            final String error = errBAIS.toString();
            assertTrue("An error message is printed", error.isEmpty());
            assertEquals("Unexpected exit code", 0, shell.getExitStatus());

        } finally {
            System.setErr(standardConsoleError);

            preferenceFile.delete();

            PreferenceFileManager.reset();
        }
    }
}
