/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.URL_LONG_OPTION;
import static org.ow2.petals.cli.shell.command.Deploy.BULK_SHORT_OPTION;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import org.junit.Test;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.cli.api.exception.ShellException;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class DeployTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws CommandException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Deploy command = new Deploy();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     * 
     * @see <a
     *      href="http://jira.petalslink.com/browse/PETALSESBCLI-45">PETALSESBCLI-45</a>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Deploy command = new Deploy();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] {});
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingOptionsException.MISSING_OPTIONS));
        }
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_1() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Deploy command = new Deploy();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    @Test()
    public void testDeployOneArtifact() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ArtifactAdministration artifactAdministration = PetalsAdministrationFactory
                .newInstance().newArtifactAdministration();
        try {
            Deploy command = new Deploy();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "--" + URL_LONG_OPTION, componentURL.toString() });
            assertEquals(1, artifactAdministration.listArtifacts().size());
        } finally {
            artifactAdministration.stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testErrorWithMoreThanOneURL() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ArtifactAdministration artifactAdministration = PetalsAdministrationFactory
                .newInstance().newArtifactAdministration();
        try {
            Deploy command = new Deploy();
            command.setShell(dummyShell.getShell());
            try {
                command.execute(new String[] { "--" + URL_LONG_OPTION, componentURL.toString(),
                        saURL.toString() });

                fail("Exception 'CommandTooManyArgumentsException' is not thrown");
            } catch (final CommandTooManyArgumentsException e) {
                assertTrue(
                        "Error label is missing.",
                        e.getMessage().startsWith(
                                CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
            } catch (final CommandException e) {
                fail("Exception 'CommandTooManyArgumentsException' is not thrown");
            }
        } finally {
            artifactAdministration.stopAndUndeployAllArtifacts();
        }
    }

    @Test()
    public void testDeployOneArtifactWithPropertiesFile() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        URL propertiesFile = Thread.currentThread().getContextClassLoader()
                .getResource("test.properties");
        DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ArtifactAdministration artifactAdministration = PetalsAdministrationFactory
                .newInstance().newArtifactAdministration();
        try {
            Deploy command = new Deploy();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "--" + URL_LONG_OPTION, componentURL.toString(), "-f",
                    propertiesFile.toString() });
            assertEquals(1, artifactAdministration.listArtifacts().size());
        } finally {
            artifactAdministration.stopAndUndeployAllArtifacts();
        }
    }

    @Test()
    public void testDeployOneArtifactWithInlinedProperties() throws ShellException,
            CommandException, MalformedURLException, ArtifactAdministrationException,
            DuplicatedServiceException, MissingServiceException {
        DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ArtifactAdministration artifactAdministration = PetalsAdministrationFactory
                .newInstance().newArtifactAdministration();
        try {
            Deploy command = new Deploy();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "--" + URL_LONG_OPTION, componentURL.toString(), "-D",
                    "key1=value1,key2=value2" });
            assertEquals(1, artifactAdministration.listArtifacts().size());
        } finally {
            artifactAdministration.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>the url of the artifact (component) to deploy is correctly provided,</li>
     * <li>configuration properties option is set without value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Deploy command = new Deploy();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "--" + URL_LONG_OPTION, componentURL.toString(),
                    "-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION
                    + "' has a missing argument.",
                    Deploy.INLINED_PROPERTIES_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(Deploy.INLINED_PROPERTIES_SHORT_OPTION));
        }
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>the url of the artifact (component) to deploy is correctly provided,</li>
     * <li>configuration properties option is set with value containing only a
     * property name, no property value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws CommandException {
        final String unparsableProperty = "key2;value2";
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Deploy command = new Deploy();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "--" + URL_LONG_OPTION, componentURL.toString(),
                    "-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION,
                    "key1=value1," + unparsableProperty });
            fail("CommandUnparsableArgumentException is not thrown");
        } catch (final CommandUnparsableArgumentException e) {
            assertTrue("The option is missing.",
                    e.getMessage().contains("-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION));
            assertTrue("The unparsable value is missing.",
                    e.getMessage().contains(unparsableProperty));
        }
    }

    @Test()
    public void testBulkDeploy() throws ShellException, CommandException, MalformedURLException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            URISyntaxException {
        DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ArtifactAdministration artifactAdministration = PetalsAdministrationFactory
                .newInstance().newArtifactAdministration();
        try {
            Deploy command = new Deploy();
            command.setShell(dummyShell.getShell());
            File file = new File(componentURL.toURI()).getParentFile();
            assertTrue(file.isDirectory());
            command.execute(new String[] { "-" + BULK_SHORT_OPTION, file.getPath() });
            assertEquals(5, artifactAdministration.listArtifacts().size());
        } finally {
            artifactAdministration.stopAndUndeployAllArtifacts();
        }
    }

    @Test(expected = CommandException.class)
    public void testBulkDeployWithNonFileTypeURL() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException {
        DummyShellWrapper dummyShell = new DummyShellWrapper();
        Deploy command = new Deploy();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-" + BULK_SHORT_OPTION, "http://myfile.zip" });
        } catch (CommandException e) {
            assertEquals("Directory deployment is only allowed for file protocol", e.getMessage());
            throw e;
        }
    }

}
