/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;

import org.junit.Test;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.shell.PetalsInteractiveCliTest;

/**
 * Unit test of {@link Disconnect}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DisconnectTest extends AbstractArtifactCommandTest {

    /**
     * Assertion exception used by threads
     */
    private volatile AssertionError assertion;

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws CommandException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Disconnect command = new Disconnect();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "disconnect", where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the command 'disconnect' is executed before to exit.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>the connection is established before to execute 'disconnect',</li>
     * </li>the connection is released after 'disconnect', and before
     * 'exit'.</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testExecute_1() throws CommandException, DuplicatedServiceException,
            MissingServiceException, ContainerAdministrationException, IOException,
            InterruptedException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            this.assertion = null;
            final ContainerAdministration containerAdministration = PetalsAdministrationFactory
                    .newInstance().newContainerAdministration();
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            final String error1 = errBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("An error occurs", error1.isEmpty());
                            assertTrue("Invalid prompt", output1.contains("petals-cli>"));
                            assertFalse("Invalid prompt", output1.contains("localhost:7700"));

                            // Connect
                            pos.write("connect\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String output2 = outBAIS.toString().substring(output1.length());
                            final String error2 = errBAIS.toString().substring(error1.length());
                            assertFalse("Prompt is empty", output2.isEmpty());
                            assertTrue("An error occurs", error2.isEmpty());
                            assertTrue(
                                    "Confirmation message is missing",
                                    output2.contains("Would you like to connect to petals:*****@127.0.0.1:7700? (y/n)"));

                            // Confirmation
                            pos.write("y\n".getBytes());
                            // Wait the execution of the confirmation
                            Thread.sleep(1000);
                            final String output3 = outBAIS.toString().substring(
                                    output1.length() + output2.length());
                            final String error3 = errBAIS.toString().substring(
                                    error1.length() + error2.length());
                            assertFalse("Prompt is empty", output3.isEmpty());
                            assertTrue("An error occurs", error3.isEmpty());
                            assertTrue("Petals CLI is not connected",
                                    output3.contains("petals-cli@127.0.0.1:7700>"));
                            assertTrue("Connection not established",
                                    containerAdministration.isConnected());

                            // Enter the command
                            pos.write("disconnect\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error4 = errBAIS.toString().substring(
                                    error1.length() + error2.length() + error3.length());
                            assertTrue("An error occurs", error4.isEmpty());
                            assertFalse("Connection not disconnected",
                                    containerAdministration.isConnected());

                            // Enter the command
                            pos.write("exit\n".getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error5 = errBAIS.toString().substring(
                                    error1.length() + error2.length() + error3.length()
                                            + error4.length());
                            assertTrue("An error occurs", error5.isEmpty());

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        DisconnectTest.this.assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-h", "localhost", "-n",
                                "7700", "-u", "petals", "-p", "petals", "-C" });

                        assertEquals("invalid exit code", 0, exitCode);

                    } catch (final AssertionError e) {
                        DisconnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);
        }
    }

}
