/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Collections;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.artifact.ArtifactException;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class ListArtifactsTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against no deployed
     * artifact.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>the output is empty</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecuteWithoutArtifactsInRegistry() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());

        command.execute(new String[] {});
        assertEquals("", dummyShell.getOut().toString());
    }

    @Test
    public void testExecuteWithTwoArtifactsInRegistry() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            String slName = ArtifactUtils.createArtifact(slURL).getName();
            String componentName = ArtifactUtils.createArtifact(componentURL).getName();
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(saURL);
            String saName = sa.getName();

            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());

            command.execute(new String[] {});
            StringBuilder expected = new StringBuilder(slName + " SL\n" + componentName
                    + " STARTED BC\n" + saName + " STARTED SA\n");

            List<ServiceUnit> serviceUnits = sa.getServiceUnits();

            // Alphabetical sorting
            Collections.sort(serviceUnits, command.artifactNameComparator);
            for (ServiceUnit su : serviceUnits) {
                expected.append(su.getName() + " SU\n");

            }
            assertEquals(expected.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number
     * arguments, where the artifact regexp is provided without the option:
     * <code>list .*(EIP|eip|soap).*</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_0() throws CommandException,
            ArtifactAdministrationException, ArtifactException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { ".*(EIP|eip|soap).*" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number
     * arguments, where the artifact type is provided without the option:
     * <code>list SU</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_1() throws CommandException,
            ArtifactAdministrationException, ArtifactException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "SU" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number
     * arguments, where the artifact pattern is correctly provided and the
     * artifact type is provided without the option:
     * <code>list -p .*(EIP|eip|soap).* SU</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_2() throws CommandException,
            ArtifactAdministrationException, ArtifactException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-" + ListArtifacts.REGEX_SHORT_OPTION,
                    ".*(EIP|eip|soap).*", "SU" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number
     * arguments, where the artifact type is correctly provided and the artifact
     * pattern is provided without the option:
     * <code>list -t SU .*(EIP|eip|soap).*</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_3() throws CommandException,
            ArtifactAdministrationException, ArtifactException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU",
                    ".*(EIP|eip|soap).*" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number
     * arguments, where the artifact pattern is provided without the option and
     * the artifact type is correctly provided:
     * <code>list .*(EIP|eip|soap).* -t SU </code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_4() throws CommandException,
            ArtifactAdministrationException, ArtifactException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { ".*(EIP|eip|soap).*",
                    "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * Test of the command "list" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_5() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact, where:
     * <ul>
     * <li>the artifact pattern is provided,</li>
     * <li>several JBI artifacts are deployed (1 SL, 1 BC matching the regexp, 1
     * SA including 8 SUs with 4 matching the regex.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error is throw,</li>
     * <li>the 5 artifacts matching the regexp are listed,</li>
     * <li>the JBI component is displayed first, next the 4 artifacts are
     * displayed in the alphabetical order</li>
     * </ul>
     * </p>
     */
    @Test
    public void testListArtifactWithRegexFilter_0() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            String componentName = ArtifactUtils.createArtifact(componentURL).getName();
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(saURL);

            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());

            command.execute(new String[] { "-" + ListArtifacts.REGEX_SHORT_OPTION,
                    ".*(EIP|eip|soap).*" });

            final String output = dummyShell.getOut().toString();
            final int idxComponent = output.indexOf(componentName);
            assertTrue("Component not found in the list", idxComponent > -1);

            List<ServiceUnit> serviceUnits = sa.getServiceUnits();
            final int[] idxSu = new int[serviceUnits.size()];
            int i = 0;
            Collections.sort(serviceUnits, command.artifactNameComparator);
            for (final ServiceUnit serviceUnit : serviceUnits) {
                if (serviceUnit.getName().contains("-EIP-")) {
                    idxSu[i++] = output.indexOf(serviceUnit.getName());
                    if (i == 1) {
                        assertTrue("SU is not at the right place", idxSu[0] > idxComponent);
                    } else {
                        assertTrue("SU is not at the right place", idxSu[i - 1] > idxSu[i - 2]);
                    }
                }
            }
            assertEquals("SU list has not the right size", 4, i);
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }

    }

    @Test
    public void testListArtifactWithSAFilter() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(saURL);

            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SA" });

            assertEquals(sa.getName() + " STARTED SA\n", dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testListArtifactWithSUFilter() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(saURL);

            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());

            command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU" });
            StringBuilder expected = new StringBuilder();

            List<ServiceUnit> serviceUnits = sa.getServiceUnits();

            // Alphabetical sorting
            Collections.sort(serviceUnits, command.artifactNameComparator);
            for (ServiceUnit su : serviceUnits) {
                expected.append(su.getName() + " SU\n");

            }
            assertEquals(expected.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testListArtifactWithBCFilter() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            String componentName = ArtifactUtils.createArtifact(componentURL).getName();
            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "BC" });

            assertEquals(componentName + " STARTED BC\n", dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testListArtifactWithSEFilter() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SE" });

            assertEquals("", dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testListArtifactWithSLFilter() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            String slName = ArtifactUtils.createArtifact(slURL).getName();
            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SL" });

            assertEquals(slName + " SL\n", dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testListArtifactWithRegexAndArtifactTypeFilter() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(slURL, componentURL, saURL);

        try {
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(saURL);

            final DummyShellWrapper dummyShell = new DummyShellWrapper();
            final ListArtifacts command = new ListArtifacts();
            command.setShell(dummyShell.getShell());

            command.execute(new String[] { "-" + ListArtifacts.REGEX_SHORT_OPTION, ".*EIP.*",
                    "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU" });
            StringBuilder expected = new StringBuilder();

            List<ServiceUnit> serviceUnits = sa.getServiceUnits();

            // Alphabetical sorting
            Collections.sort(serviceUnits, command.artifactNameComparator);
            for (ServiceUnit su : serviceUnits) {
                if (su.getName().contains("EIP")) {
                    expected.append(su.getName() + " SU\n");
                }
            }
            assertEquals(expected.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }
}
