/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.ow2.petals.admin.api.artifact.ArtifactException;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.RegistryRegexpPatternException;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;

/**
 * Unit test of {@link ListRegistry}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ListRegistryTest extends AbstractArtifactCommandTest {

    private final static String INVALID_REGEXP = "{http://petals.ow2.org/}FaultIntegrationItf";

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListRegistry command = new ListRegistry();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * Test of the command "registry-list" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListRegistry command = new ListRegistry();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "registry-list" against argument management, where:
     * <ul>
     * <li>the regexp used to filter interfaces contains an error</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the regexp error.
     * </p>
     */
    @Test
    public void testArgumentsError_1() throws CommandException, ArtifactAdministrationException,
            ArtifactException {
        // TODO: prefer to deploy a SA to fill the registry instead of using
        // data defined in test classes petals-admin-api.

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListRegistry command = new ListRegistry();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-i", INVALID_REGEXP });
            fail("RegistryRegexpPatternException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not RegistryRegexpPatternException",
                    e.getCause() instanceof RegistryRegexpPatternException);
            assertTrue("Invalid regexp is not displayed", e.getMessage().contains(INVALID_REGEXP));
        }
    }

    /**
     * Test of the command "registry-list" against argument management, where:
     * <ul>
     * <li>the regexp used to filter services contains an error</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the regexp error.
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws CommandException, ArtifactAdministrationException,
            ArtifactException {
        // TODO: prefer to deploy a SA to fill the registry instead of using
        // data defined in test classes petals-admin-api.

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListRegistry command = new ListRegistry();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-s", INVALID_REGEXP });
            fail("RegistryRegexpPatternException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not RegistryRegexpPatternException",
                    e.getCause() instanceof RegistryRegexpPatternException);
            assertTrue("Invalid regexp is not displayed", e.getMessage().contains(INVALID_REGEXP));
        }
    }

    /**
     * Test of the command "registry-list" against argument management, where:
     * <ul>
     * <li>the regexp used to filter endpoints contains an error</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the regexp error.
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws CommandException, ArtifactAdministrationException,
            ArtifactException {
        // TODO: prefer to deploy a SA to fill the registry instead of using
        // data defined in test classes petals-admin-api.

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final ListRegistry command = new ListRegistry();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-e", INVALID_REGEXP });
            fail("RegistryRegexpPatternException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not RegistryRegexpPatternException",
                    e.getCause() instanceof RegistryRegexpPatternException);
            assertTrue("Invalid regexp is not displayed", e.getMessage().contains(INVALID_REGEXP));
        }
    }
}
