/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.ARTIFACT_SHORT_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;
import static org.ow2.petals.cli.shell.command.Show.IDENTIFIER_LBL;
import static org.ow2.petals.cli.shell.command.Show.SEPARATOR;
import static org.ow2.petals.cli.shell.command.Show.SERVICEUNITS_LBL;
import static org.ow2.petals.cli.shell.command.Show.STATE_LBL;
import static org.ow2.petals.cli.shell.command.Show.TYPE_LBL;
import static org.ow2.petals.cli.shell.command.Show.VERSION_LBL;

import java.util.List;

import org.junit.Test;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactException;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactTypeIsNeededException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;

public class ShowTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws CommandException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Show command = new Show();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "show" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Show command = new Show();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "show" without arguments.
     */
    @Test
    public void testExecuteWithoutArgument() throws CommandException {
        DummyShellWrapper dummyShell = new DummyShellWrapper();
        Show command = new Show();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] {});
        } catch (CommandException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "show" to display information about a component
     * accessed by URL.
     */
    @Test
    public void testShowComponentInformation() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + URL_SHORT_OPTION, this.componentURL.toString() });
            StringBuilder b = new StringBuilder();
            b.append(IDENTIFIER_LBL + SEPARATOR + a.getName()
                    + System.getProperty("line.separator"));
            b.append(TYPE_LBL + SEPARATOR + a.getType() + System.getProperty("line.separator"));
            b.append(VERSION_LBL + SEPARATOR + Show.UNKNOWN_VERSION
                    + System.getProperty("line.separator"));
            b.append(STATE_LBL + SEPARATOR + ArtifactState.State.STARTED
                    + System.getProperty("line.separator"));

            assertEquals(b.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>information are correctly displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_0() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String identifier = a.getName();
            String type = a.getType();
            command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, identifier, type });
            StringBuilder expectedOutput = new StringBuilder();
            expectedOutput.append(IDENTIFIER_LBL + SEPARATOR + a.getName()
                    + System.getProperty("line.separator"));
            expectedOutput.append(TYPE_LBL + SEPARATOR + a.getType()
                    + System.getProperty("line.separator"));
            expectedOutput.append(VERSION_LBL + SEPARATOR + Show.UNKNOWN_VERSION
                    + System.getProperty("line.separator"));
            expectedOutput.append(STATE_LBL + SEPARATOR + ArtifactState.State.STARTED
                    + System.getProperty("line.separator"));

            assertEquals(expectedOutput.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>information are correctly displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_1() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String identifier = a.getName();
            command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, identifier });
            StringBuilder expectedOutput = new StringBuilder();
            expectedOutput.append(IDENTIFIER_LBL + SEPARATOR + a.getName()
                    + System.getProperty("line.separator"));
            expectedOutput.append(TYPE_LBL + SEPARATOR + a.getType()
                    + System.getProperty("line.separator"));
            expectedOutput.append(VERSION_LBL + SEPARATOR + Show.UNKNOWN_VERSION
                    + System.getProperty("line.separator"));
            expectedOutput.append(STATE_LBL + SEPARATOR + ArtifactState.State.STARTED
                    + System.getProperty("line.separator"));

            assertEquals(expectedOutput.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing artifact type.
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_2()
            throws ArtifactAdministrationException, ArtifactException, CommandException,
            DuplicatedServiceException, MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            this.deployStartArtifact(this.saSoapidURL);
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(this.saSoapidURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String identifier = a.getName();
            assertEquals("Artifacts have not the same identifier", sa.getName(), identifier);
            try {
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, identifier });
                fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactTypeIsNeededException",
                        e.getCause() instanceof ArtifactTypeIsNeededException);
            }
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>information are correctly displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_3() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            this.deployStartArtifact(this.saSoapidURL);
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(this.saSoapidURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String identifier = a.getName();
            assertEquals("Artifacts have not the same identifier", sa.getName(), identifier);
            String type = a.getType();
            command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, identifier, type });
            StringBuilder expectedOutput = new StringBuilder();
            expectedOutput.append(IDENTIFIER_LBL + SEPARATOR + a.getName()
                    + System.getProperty("line.separator"));
            expectedOutput.append(TYPE_LBL + SEPARATOR + a.getType()
                    + System.getProperty("line.separator"));
            expectedOutput.append(VERSION_LBL + SEPARATOR + Show.UNKNOWN_VERSION
                    + System.getProperty("line.separator"));
            expectedOutput.append(STATE_LBL + SEPARATOR + ArtifactState.State.STARTED
                    + System.getProperty("line.separator"));

            assertEquals(expectedOutput.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_4()
            throws ArtifactAdministrationException, ArtifactException, CommandException,
            DuplicatedServiceException, MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            this.deployStartArtifact(this.saSoapidURL);
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(this.saSoapidURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();
            assertEquals("Artifacts have not the same identifier", sa.getName(), a.getName());
            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String type = a.getType();
            try {
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, type });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_5()
            throws ArtifactAdministrationException, ArtifactException, CommandException,
            DuplicatedServiceException, MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            this.deployStartArtifact(this.saSoapidURL);
            ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(this.saSoapidURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String identifier = a.getName();
            assertEquals("Artifacts have not the same identifier", sa.getName(), identifier);
            try {
                assertFalse("The artifact is a SE", "SE".equals(a.getType()));
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, identifier, "SE" });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_6()
            throws ArtifactAdministrationException, ArtifactException, CommandException,
            DuplicatedServiceException, MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String identifier = a.getName();
            try {
                assertFalse("The artifact is a SE", "SE".equals(a.getType()));
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, identifier, "SE" });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "show" to display information about a component
     * accessed by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testShowArtifactInformationByNameAndIdentifier_7()
            throws ArtifactAdministrationException, ArtifactException, CommandException,
            DuplicatedServiceException, MissingServiceException {
        this.deployStartArtifact(this.componentURL);
        try {
            Component a = (Component) ArtifactUtils.createArtifact(this.componentURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();
            Show command = new Show();
            command.setShell(dummyShell.getShell());
            String type = a.getType();
            try {
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, type });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * Test of the command "show" to display information about a service
     * assembly accessed by URL.
     */
    @Test
    public void testShowServiceAssemblyInformation() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(this.saURL);
        try {
            ServiceAssembly a = (ServiceAssembly) ArtifactUtils.createArtifact(this.saURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + URL_SHORT_OPTION, this.saURL.toString() });

            StringBuilder b = new StringBuilder();
            b.append(IDENTIFIER_LBL + SEPARATOR + a.getName()
                    + System.getProperty("line.separator"));
            b.append(VERSION_LBL + SEPARATOR + Show.UNKNOWN_VERSION
                    + System.getProperty("line.separator"));
            b.append(STATE_LBL + SEPARATOR + ArtifactState.State.STARTED
                    + System.getProperty("line.separator"));
            StringBuilder expectedSus = new StringBuilder();
            List<ServiceUnit> sus = a.getServiceUnits();
            for (int i = 0; i < sus.size(); i++) {
                expectedSus.append(sus.get(i).getName());
                if (i != sus.size() - 1) {
                    expectedSus.append(" ");
                }
            }
            b.append(SERVICEUNITS_LBL + SEPARATOR + expectedSus
                    + System.getProperty("line.separator"));

            assertEquals(b.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * Test of the command "show" to display information about a shared library
     * accessed by artifact type and artifact identifier.
     */
    @Test
    public void testShowSharedLibraryInformation() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(this.slURL);
        try {
            Artifact a = ArtifactUtils.createArtifact(this.slURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + URL_SHORT_OPTION, this.slURL.toString() });

            StringBuilder b = new StringBuilder();
            b.append(IDENTIFIER_LBL + SEPARATOR + a.getName()
                    + System.getProperty("line.separator"));
            b.append(VERSION_LBL + SEPARATOR + Show.UNKNOWN_VERSION
                    + System.getProperty("line.separator"));

            assertEquals(b.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }

    /**
     * Test of the command "show" to display <b>extended</b> information about a
     * shared library accessed by artifact type and artifact identifier.
     */
    @Test
    public void testShowSharedLibraryExtendedInformation() throws CommandException,
            ArtifactAdministrationException, ArtifactException, DuplicatedServiceException,
            MissingServiceException {
        this.deployStartArtifact(this.slURL);
        try {
            Artifact a = ArtifactUtils.createArtifact(this.slURL);
            DummyShellWrapper dummyShell = new DummyShellWrapper();

            Show command = new Show();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-e", "-" + URL_SHORT_OPTION, this.slURL.toString() });

            StringBuilder b = new StringBuilder();
            b.append(IDENTIFIER_LBL + SEPARATOR + a.getName()
                    + System.getProperty("line.separator"));
            b.append(VERSION_LBL + SEPARATOR + Show.UNKNOWN_VERSION
                    + System.getProperty("line.separator"));

            assertEquals(b.toString(), dummyShell.getOut().toString());
        } finally {
            PetalsAdministrationFactory.newInstance().newArtifactAdministration()
                    .stopAndUndeployAllArtifacts();
        }
    }
}