/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.ARTIFACT_SHORT_OPTION;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;

import java.net.MalformedURLException;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactTypeIsNeededException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.exception.ShellException;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class StopArtifactTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws CommandException {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final StopArtifact command = new StopArtifact();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "stop-artifact" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final StopArtifact command = new StopArtifact();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    @Test
    public void testStopOneArtifactByURL() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();

        admin.deployAndStartArtifact(componentURL);
        try {
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals(1, artifacts.size());
            Artifact artifact = admin.getArtifactInfo(componentURL);
            assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + URL_SHORT_OPTION, componentURL.toString() });
            artifact = admin.getArtifactInfo(componentURL);
            assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) artifact).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_0() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            Artifact artifact = admin.getArtifactInfo(componentURL);
            String name = artifact.getName();
            String type = artifact.getType();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if component is deployed", 1, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, name, type });
            artifact = admin.getArtifactInfo(componentURL);
            assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) artifact).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_1() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            Artifact artifact = admin.getArtifactInfo(componentURL);
            String name = artifact.getName();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if component is deployed", 1, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, name });
            artifact = admin.getArtifactInfo(componentURL);
            assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) artifact).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing artifact type</li>
     * <li>the artifacts are not stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_2() throws ShellException, MalformedURLException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            admin.deployAndStartArtifact(saSoapidURL);
            Artifact artifact = admin.getArtifactInfo(componentURL);
            String name = artifact.getName();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if artifacts are deployed", 2, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            try {
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, name });
                fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactTypeIsNeededException",
                        e.getCause() instanceof ArtifactTypeIsNeededException);
            }
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>the right artifact is stopped</li>
     * <li>the other artifact is not stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_3() throws ShellException, CommandException,
            MalformedURLException, ArtifactAdministrationException, DuplicatedServiceException,
            MissingServiceException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            admin.deployAndStartArtifact(saSoapidURL);
            Artifact artifact = admin.getArtifactInfo(componentURL);
            String name = artifact.getName();
            final String type = artifact.getType();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if artifacts are deployed", 2, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, name, type });
            assertEquals(ArtifactState.State.STOPPED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifacts are always started</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_4() throws ShellException, MalformedURLException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            admin.deployAndStartArtifact(saSoapidURL);
            Artifact artifact = admin.getArtifactInfo(componentURL);
            final String type = artifact.getType();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if artifacts are deployed", 2, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            try {
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, type });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifacts are always started</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_5() throws ShellException, MalformedURLException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            admin.deployAndStartArtifact(saSoapidURL);
            Artifact artifact = admin.getArtifactInfo(componentURL);
            String name = artifact.getName();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if artifacts are deployed", 2, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            try {
                assertFalse("The artifact is a SE", "SE".equals(artifact.getType()));
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, name, "SE" });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(saSoapidURL)).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifact is always started</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_6() throws ShellException, MalformedURLException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            Artifact artifact = admin.getArtifactInfo(componentURL);
            String name = artifact.getName();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if component is deployed", 1, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            try {
                assertFalse("The artifact is a SE", "SE".equals(artifact.getType()));
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, name, "SE" });
                fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifact is always started</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_7() throws ShellException, MalformedURLException,
            ArtifactAdministrationException, DuplicatedServiceException, MissingServiceException,
            CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();

        ArtifactAdministration admin = PetalsAdministrationFactory.newInstance()
                .newArtifactAdministration();
        admin.deployAndStartArtifact(componentURL);
        try {
            Artifact artifact = admin.getArtifactInfo(componentURL);
            String type = artifact.getType();
            List<Artifact> artifacts = admin.listArtifacts();
            assertEquals("Check if component is deployed", 1, artifacts.size());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
            StopArtifact command = new StopArtifact();
            command.setShell(dummyShell.getShell());
            try {
                command.execute(new String[] { "-" + ARTIFACT_SHORT_OPTION, type });
                fail("ArtifactNotFoundException is not thrown through CommandException.");
            } catch (final CommandException e) {
                assertNotNull("CommandException cause is missing.", e.getCause());
                assertTrue("CommandException cause is not ArtifactNotFoundException",
                        e.getCause() instanceof ArtifactNotFoundException);
            }
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(componentURL)).getState());
        } finally {
            admin.stopAndUndeployAllArtifacts();
        }
    }

    @Test
    public void testStopManyArtifactThrownException() throws ShellException, MalformedURLException,
            ArtifactAdministrationException, CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        StopArtifact command = new StopArtifact();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-" + URL_SHORT_OPTION, componentURL.toString(),
                    saURL.toString() });

            fail("Exception 'CommandTooManyArgumentsException' is not thrown");
        } catch (final CommandTooManyArgumentsException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
        } catch (final CommandException e) {
            fail("Exception 'CommandTooManyArgumentsException' is not thrown");
        }
    }
}
