/**
 * Copyright (c) 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;

import org.junit.Test;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingYesFlagException;
import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.exception.DuplicatedCommandException;
import org.ow2.petals.cli.shell.PetalsCli;
import org.ow2.petals.cli.shell.PetalsInteractiveCliTest;
import org.ow2.petals.cli.shell.ShellFactory;

/**
 * Unit test of {@link Stop}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class StopTest extends AbstractArtifactCommandTest {

    /**
     * The shell {@link PetalsCli} establishes automatically a connection. Here,
     * the default connection parameters.
     */
    private final ConnectionParameters defaultConnectionParameters = new AuthenticatedConnectionParameters(
            "localhost", 7700, "petals", "petals", null);
    
    private final static String EXPECTED_CMD_USAGE = "usage: stop [-s] [-y]"
            + System.getProperty("line.separator");

    /**
     * Assertion exception used by threads
     */
    private volatile AssertionError assertion;

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {

        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Stop command = new Stop();
        command.setShell(dummyShell.getShell());
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertEquals("Unexpected command usage", EXPECTED_CMD_USAGE, usage);
    }

    /**
     * Test of the command "stop" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_00() throws CommandException {
        final DummyShellWrapper dummyShell = new DummyShellWrapper();
        final Stop command = new Stop();
        command.setShell(dummyShell.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "stop" against argument management, where:
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with connection parametern</li>
     * <li>without yes flag,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing yes flag.
     * </p>
     */
    @Test
    public void testArgumentsError_01() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException {

        final PrintStream standardConsoleError = System.err;

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            final Stop command = new Stop();
            final PetalsCli shell = new PetalsCli(new String[] { command.getName(),
                    "-" + Stop.SHUTDOWN_SHORT_OPTION }, false, false);
            shell.registersCommand(command);
            shell.setConnectionParameters(this.defaultConnectionParameters);
            command.setShell(shell);

            shell.run();

            final String error = errBAIS.toString();
            assertFalse("No error message print", error.isEmpty());
            assertTrue("It's not the expected error message.",
                    error.contains(CommandMissingYesFlagException.MISSING_YESFLAG));

        } finally {
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test of the command "stop" against argument management, where:
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testArguments_00() {

        final PrintStream standardConsoleError = System.err;

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final int exitCode = main.run(new String[] { "-" + ShellFactory.YESFLAG_SHORT_OPTION,
                    "-" + ShellFactory.COMMAND_SHORT_OPTION, "--", new Stop().getName(),
                    "-" + Stop.SHUTDOWN_SHORT_OPTION });

            final String error = errBAIS.toString();
            assertEquals("invalid exit code", 0, exitCode);
            assertTrue("An error message is print", error.isEmpty());

        } finally {
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test of the command "stop" against argument management, where:
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using a non interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: No error occurs about missing yes flag
     * </p>
     */
    @Test
    public void testArguments_01() {

        final PrintStream standardConsoleError = System.err;

        try {
            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();

            final int exitCode = main.run(new String[] { "-" + ShellFactory.COMMAND_SHORT_OPTION,
                    "--", new Stop().getName(), "-" + Stop.SHUTDOWN_SHORT_OPTION,
                    "-" + Stop.YESFLAG_SHORT_OPTION, });

            final String error = errBAIS.toString();
            assertEquals("invalid exit code", 0, exitCode);
            assertTrue("An error message is print", error.isEmpty());

        } finally {
            System.setErr(standardConsoleError);
        }
    }

    /**
     * Test of the command "stop" against argument management, where:
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>a confirmation message is displayed,</li>
     * <li>after confirmation, the command is executed displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_02() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));
            
            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {
    
                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));
                            // Enter the command
                            pos.write(String.format("%s -s%n", new Stop().getName()).getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error1 = errBAIS.toString();
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertTrue("An error message is print", error1.isEmpty());
                            assertFalse("No confirmation message print", output2.isEmpty());
                            assertTrue("Unexpected confirmation message",
                                    output2.endsWith(Stop.SHUTDOWN_CONFIRMATION_MSG));
                            
                            // Enter confirmation
                            pos.write(String.format("y%n").getBytes());
                            pos.flush();
                            // Wait the execution of the command
                            Thread.sleep(250);
                            final String error2 = errBAIS.toString().substring(error1.length());
                            final String output3 = outBAIS.toString().substring(output2.length());
                            assertTrue("An error message is print", error2.isEmpty());
                            assertTrue("Unexpected prompt", output3.endsWith("petals-cli>"));
    
                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        assertion = e;
                    }
                }
            };
    
            final Runnable petalsCli = new Runnable() {
    
                @Override
                public void run() {
    
                    try {
                        final int exitCode = main.run(new String[] { "-C" });
    
                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        assertion = e;
                    }
    
                }
            };
    
            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();
    
            if (this.assertion != null) {
                throw this.assertion;
            }
        }
        finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);
        }
    }

    /**
     * Test of the command "stop" against argument management, where:
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command line level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArguments_02() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {

            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));
                            // Enter the command
                            pos.write(String.format("%s -s%n", new Stop().getName()).getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error1 = errBAIS.toString();
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertTrue("An error message is print", error1.isEmpty());
                            assertTrue(
                                    "Unexpected prompt (probably a confirmation message is printed)",
                                    output2.endsWith("petals-cli>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] {
                                "-" + ShellFactory.YESFLAG_SHORT_OPTION, "-C" });

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);
        }
    }

    /**
     * Test of the command "stop" against argument management, where:
     * <ul>
     * <li>a shutdown of a container is asked,</li>
     * <li>with the yes flag at command level,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no confirmation message is displayed,</li>
     * <li>the command is executed displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArguments_03() throws CommandException, DuplicatedCommandException,
            IllegalArgumentException, InterruptedException, IOException {

        final PrintStream standardConsoleOutput = System.out;
        final PrintStream standardConsoleError = System.err;
        final InputStream standardConsoleInput = System.in;

        try {

            // Enable the hack for Windows?
            PetalsInteractiveCliTest.hackForWindows(true);

            final ByteArrayOutputStream errBAIS = new ByteArrayOutputStream();
            System.setErr(new PrintStream(errBAIS));
            final ByteArrayOutputStream outBAIS = new ByteArrayOutputStream();
            System.setOut(new PrintStream(outBAIS));
            final PipedOutputStream pos = new PipedOutputStream();
            System.setIn(new PipedInputStream(pos));

            // The main class MUST be instanced after to have set the console
            // streams !!
            final Main main = new Main();
            this.assertion = null;
            final Runnable writer = new Runnable() {

                @Override
                public void run() {
                    try {
                        try {
                            // Wait that Petals CLI starts
                            Thread.sleep(1000);

                            final String output1 = outBAIS.toString();
                            assertFalse("Prompt is empty", output1.isEmpty());
                            assertTrue("Petals CLI is connected", output1.contains("petals-cli>"));
                            // Enter the command
                            pos.write(String.format("%s -s -y%n", new Stop().getName()).getBytes());
                            // Wait the execution of the command
                            Thread.sleep(1000);
                            final String error1 = errBAIS.toString();
                            final String output2 = outBAIS.toString().substring(output1.length());
                            assertTrue("An error message is print", error1.isEmpty());
                            assertTrue(
                                    "Unexpected prompt (probably a confirmation message is printed)",
                                    output2.endsWith("petals-cli>"));

                        } catch (final Exception e) {
                            fail("An exception occurs: " + e.getMessage());
                        }
                    } catch (final AssertionError e) {
                        assertion = e;
                    }
                }
            };

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = main.run(new String[] { "-C" });

                        assertEquals("invalid exit code", 0, exitCode);
                    } catch (final AssertionError e) {
                        assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            final Thread writerThread = new Thread(writer, "writerThread");
            petalsCliThread.start();
            writerThread.start();
            writerThread.join();
            pos.write(String.format("exit%n").getBytes());
            pos.close();
            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            System.setOut(standardConsoleOutput);
            System.setErr(standardConsoleError);
            System.setIn(standardConsoleInput);

            // Disable the hack for Windows
            PetalsInteractiveCliTest.hackForWindows(false);
        }
    }
}
